<%-- $Id: edit_eventtype.jsp 7954 2021-05-12 13:27:22Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------


	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.PlateType"
	import="net.sf.basedb.core.PlateEventType"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.BaseException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.edit.EditUtil"
	import="net.sf.basedb.clients.web.extensions.tabcontrol.TabAction"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.List"
	import="java.util.Set"
	import="java.util.HashSet"
	import="java.util.Date"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="t" uri="/WEB-INF/tab.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
final Item itemType = Item.PLATEEVENTTYPE;
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);
final int itemId = cc.getId();
final int plateTypeId = Values.getInt(request.getParameter("platetype_id"));
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl(":Edit "+itemType);
try
{
	String title = null;
	PlateEventType eventType = null;
	PlateType plateType = null;
	
	ItemSubtype currentProtocolType = null;
	boolean readCurrentProtocolType = true;
	int maxOrdinal = 0;
	
	// Load recently used items
	List<ItemSubtype> recentProtocolTypes = cc.getRecent(dc, Item.ITEMSUBTYPE);

	if (itemId == 0)
	{
		title = "Create event type";
		cc.removeObject("item");
		plateType = PlateType.getById(dc, plateTypeId);
		if (cc.getPropertyFilter("protocolType.name") != null)
		{
			currentProtocolType = Base.getFirstMatching(dc, ItemSubtype.getQuery(Item.PROTOCOL), "name", cc.getPropertyFilter("protocolType.name"));
		}
		maxOrdinal = (int)plateType.getEventTypes().count(dc);
	}
	else
	{
		eventType = PlateEventType.getById(dc, itemId);
		eventType.checkPermission(Permission.WRITE);
		plateType = eventType.getPlateType();
		cc.setObject("item", eventType);
		title = "Edit event type -- " + HTML.encodeTags(plateType.getName());
		try
		{
			currentProtocolType = eventType.getProtocolType();
		}
		catch (PermissionDeniedException ex)
		{
			readCurrentProtocolType = false;
		}
	}
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, GuiContext.item(itemType), eventType);
	ExtensionsInvoker<TabAction> invoker = EditUtil.useEditExtensions(jspContext);
	%>
	<base:page type="popup" title="<%=title%>" id="edit-page">
	<base:head scripts="tabcontrol-2.js,~eventtypes.js" styles="tabcontrol.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	<base:body>
		<h1><%=title%> <base:help tabcontrol="settings" /></h1>
		
		<form action="index.jsp?ID=<%=ID%>" method="post" name="eventtype">
		<input type="hidden" name="cmd" value="UpdateItem">
		<input type="hidden" name="platetype_id" value="<%=plateTypeId%>">

		<t:tabcontrol id="settings"
			subclass="content dialogtabcontrol"
			position="bottom" remember="<%=eventType != null%>"
			extensions="<%=invoker%>">
		<t:tab id="info" title="Event type" helpid="plateeventtype.edit">
			<table class="fullform input100 smaller">
			<tr>
				<th>Name</th>
				<td><input class="text required auto-init" data-auto-init="<%=eventType == null ? "focus-select" : "focus" %>"
					type="text" name="name" 
					value="<%=HTML.encodeTags(eventType == null ? Values.getString(cc.getPropertyValue("name"), "New event type") : eventType.getName())%>" 
					maxlength="<%=PlateEventType.MAX_NAME_LENGTH%>"></td>
				<td></td>
			</tr>
			<tr>
				<th>Ordinal</th>
				<td><input class="text required" type="text" name="ordinal" id="ordinal"
					style="width: 15em;"
					value="<%=eventType == null ? Values.getInt(cc.getPropertyValue("ordinal"), maxOrdinal+1) : eventType.getOrdinal()%>" 
					maxlength="10"></td>
				<td></td>
			</tr>
			<tr>
				<th>Protocol type</th>
				<td>
					<base:select 
						id="protocoltype_id"
						clazz="selectionlist"
						data-protocol-type-filter="<%=Item.PROTOCOL.getValue()%>"
						required="false"
						current="<%=currentProtocolType%>"
						denied="<%=!readCurrentProtocolType%>"
						recent="<%=recentProtocolTypes%>"
						newitem="<%=eventType == null%>"
					/>
				</td>
				<td></td>
			</tr>
			<tr class="dynamic">
				<th>Description</th>
				<td>
					<textarea class="text" rows="4" name="description" id="description"
						><%=HTML.encodeTags(eventType == null ? cc.getPropertyValue("description") : eventType.getDescription())%></textarea>
				</td>
				<td style="width: 20px;">
					<base:zoom textarea="description" title="Description" />
				</td>
			</tr>
			</table>
		</t:tab>
		</t:tabcontrol>
		</form>
		
		<div class="legend">
			<base:icon image="required.png" />= required information
		</div>

		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnSave" title="Save" />
			<base:button id="close" title="Cancel" />
		</base:buttongroup>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
